@@ -14,6 +14,9 @@
 //
 // SPDX-License-Identifier: Apache-2.0
 
+/*
+OPKSSH is a command-line tool that allows users to authenticate with OpenID Connect providers and generate SSH keys for secure access to servers.
+*/
 package main
 
 import (
@@ -22,27 +25,29 @@ import (
 	"fmt"
 	"log"
 	"os"
-	"os/exec"
 	"os/signal"
 	"regexp"
 	"strings"
 	"syscall"
+	"text/tabwriter"
 
-	"github.com/openpubkey/openpubkey/providers"
 	"github.com/openpubkey/opkssh/commands"
+	config "github.com/openpubkey/opkssh/commands/config"
+	"github.com/openpubkey/opkssh/internal/sysdetails"
 	"github.com/openpubkey/opkssh/policy"
 	"github.com/openpubkey/opkssh/policy/files"
+	"github.com/spf13/afero"
 	"github.com/spf13/cobra"
+	"github.com/spf13/cobra/doc"
+	"github.com/thediveo/enumflag/v2"
+	"golang.org/x/mod/semver"
+	"golang.org/x/term"
 )
 
 var (
 	// These can be overridden at build time using ldflags. For example:
-	// go build -v -o /usr/local/bin/opkssh -ldflags "-X main.issuer=http://oidc.local:${ISSUER_PORT}/ -X main.clientID=web -X main.clientSecret=secret"
+	// go build -v -o /usr/local/bin/opkssh -ldflags "-X main.Version=version"
 	Version           = "unversioned"
-	issuer            = ""
-	clientID          = ""
-	clientSecret      = ""
-	redirectURIs      = ""
 	logFilePathServer = "/var/log/opkssh.log" // Remember if you change this, change it in the install script as well
 )
 
@@ -72,16 +77,16 @@ This program allows users to:
 
 	addCmd := &cobra.Command{
 		SilenceUsage: true,
-		Use:          "add <PRINCIPAL> <EMAIL|SUB|GROUP> <ISSUER>",
+		Use:          "add <principal> <email|sub|group> <issuer>",
 		Short:        "Appends new rule to the policy file",
 		Long: `Add appends a new policy entry in the auth_id policy file granting SSH access to the specified email or subscriber ID (sub) or group.
 
 It first attempts to write to the system-wide file (/etc/opk/auth_id). If it lacks permissions to update this file it falls back to writing to the user-specific file (~/.opk/auth_id).
 
 Arguments:
-  PRINCIPAL            The target user account (requested principal).
-  EMAIL|SUB|GROUP      Email address, subscriber ID or group authorized to assume this principal. If using an OIDC group, the argument needs to be in the format of oidc:groups:<groupId>.
-  ISSUER               OpenID Connect provider (issuer) URL associated with the email/sub/group.
+  principal            The target user account (requested principal).
+  email|sub|group      Email address, subscriber ID or group authorized to assume this principal. If using an OIDC group, the argument needs to be in the format of oidc:groups:<groupId>.
+  issuer               OpenID Connect provider (issuer) URL associated with the email/sub/group.
 `,
 		Args: cobra.ExactArgs(3),
 		Example: `  opkssh add root alice@example.com https://accounts.google.com
@@ -100,6 +105,8 @@ Arguments:
 				inputIssuer = "https://login.microsoftonline.com/9188040d-6c67-4c5b-b112-36a304b66dad/v2.0"
 			case "gitlab":
 				inputIssuer = "https://gitlab.com"
+			case "hello":
+				inputIssuer = "https://issuer.hello.coop"
 			}
 
 			add := commands.AddCmd{
@@ -118,24 +125,54 @@ Arguments:
 	}
 	rootCmd.AddCommand(addCmd)
 
-	var autoRefresh bool
-	var logDir string
+	inspectCmd := &cobra.Command{
+		SilenceUsage: true,
+		Use:          "inspect <path>",
+		Short:        "Inspect and view details of an opkssh generated SSH key",
+		Example:      "  opkssh inspect ~/.ssh/id_ecdsa_sk-cert.pub",
+		RunE: func(cmd *cobra.Command, args []string) error {
+			keyPathArg := args[0]
+			inspect := commands.NewInspectCmd(keyPathArg, cmd.OutOrStdout())
+			if err := inspect.Run(); err != nil {
+				log.Println("Error executing inspect command:", err)
+				return err
+			}
+			return nil
+		},
+		Args: cobra.ExactArgs(1),
+	}
+	rootCmd.AddCommand(inspectCmd)
+
+	var autoRefreshArg bool
+	var configPathArg string
+	var createConfigArg bool
+	var configureArg bool
+	var logDirArg string
 	var providerArg string
+	var sendAccessTokenArg bool
 	var disableBrowserOpenArg bool
+	var printIdTokenArg bool
+	var printKeyArg bool
+	var keyPathArg string
+	var keyTypeArg commands.KeyType
+	var remoteRedirectURIArg string
+
 	loginCmd := &cobra.Command{
 		SilenceUsage: true,
-		Use:          "login",
+		Use:          "login [alias]",
 		Short:        "Authenticate with an OpenID Provider to generate an SSH key for opkssh",
 		Long: `Login creates opkssh SSH keys
 
 Login generates a key pair, then opens a browser to authenticate the user with the OpenID Provider. Upon successful authentication, opkssh creates an SSH public key (~/.ssh/id_ecdsa) containing the user's PK token. By default, this SSH key expires after 24 hours, after which the user must run "opkssh login" again to generate a new key.
 
 Users can then SSH into servers configured to use opkssh as the AuthorizedKeysCommand. The server verifies the PK token and grants access if the token is valid and the user is authorized per the auth_id policy.
+Arguments:
+  alias      The provider alias to use. If not specified, the OPKSSH_DEFAULT provider will be used. The aliases are defined by the OPKSSH_PROVIDERS environment variable. The format is <alias>,<issuer>,<client_id>,<client_secret>,<scopes>
 `,
 		Example: `  opkssh login
-  opkssh login --provider=<issuer>,<client_id>`,
+  opkssh login google
+  opkssh login --provider=<issuer>,<client_id>,<client_secret>,<scopes>`,
 		RunE: func(cmd *cobra.Command, args []string) error {
-
 			ctx, cancel := context.WithCancel(context.Background())
 			defer cancel()
 			sigs := make(chan os.Signal, 1)
@@ -145,38 +182,44 @@ Users can then SSH into servers configured to use opkssh as the AuthorizedKeysCo
 				cancel()
 			}()
 
-			// If LDFlags issuer is set, build providerFromLdFlags
-			var providerFromLdFlags providers.OpenIdProvider
-			if issuer != "" {
-				opts := providers.GetDefaultGoogleOpOptions()
-				opts.Issuer = issuer
-				opts.ClientID = clientID
-				opts.ClientSecret = clientSecret
-				opts.RedirectURIs = strings.Split(redirectURIs, ",")
-				providerFromLdFlags = providers.NewGoogleOpWithOptions(opts)
+			var providerAliasArg string
+			if len(args) > 0 {
+				providerAliasArg = args[0]
 			}
 
-			login := commands.NewLogin(autoRefresh, logDir, disableBrowserOpenArg, providerArg, providerFromLdFlags)
+			login := commands.NewLogin(autoRefreshArg, configPathArg, createConfigArg, configureArg, logDirArg,
+				sendAccessTokenArg, disableBrowserOpenArg, printIdTokenArg, providerArg, printKeyArg, keyPathArg,
+				providerAliasArg, keyTypeArg, remoteRedirectURIArg)
 			if err := login.Run(ctx); err != nil {
 				log.Println("Error executing login command:", err)
 				return err
 			}
 			return nil
 		},
+		Args: cobra.MaximumNArgs(1),
 	}
 
 	// Define flags for login.
-	loginCmd.Flags().BoolVar(&autoRefresh, "auto-refresh", false, "Automatically refresh PK token after login")
-	loginCmd.Flags().StringVar(&logDir, "log-dir", "", "Directory to write output logs")
-	loginCmd.Flags().BoolVar(&disableBrowserOpenArg, "disable-browser-open", false, "Set this flag to disable opening the browser. Useful for choosing the browser you want to use.")
-	loginCmd.Flags().StringVar(&providerArg, "provider", "", "OpenID Provider specification in the format: <issuer>,<client_id> or <issuer>,<client_id>,<client_secret>")
+	loginCmd.Flags().BoolVar(&autoRefreshArg, "auto-refresh", false, "Automatically refresh PK token after login")
+	loginCmd.Flags().StringVar(&configPathArg, "config-path", "", "Path to the client config file. Default: ~/.opk/config.yml on linux and %APPDATA%\\.opk\\config.yml on windows")
+	loginCmd.Flags().BoolVar(&createConfigArg, "create-config", false, "Creates a client config file if it does not exist")
+	loginCmd.Flags().BoolVar(&configureArg, "configure", false, "Apply changes to ssh config and create ~/.ssh/opkssh directory")
+	loginCmd.Flags().StringVar(&logDirArg, "log-dir", "", "Directory to write output logs")
+	loginCmd.Flags().BoolVar(&disableBrowserOpenArg, "disable-browser-open", false, "Set this flag to disable opening the browser. Useful for choosing the browser you want to use")
+	loginCmd.Flags().BoolVar(&printIdTokenArg, "print-id-token", false, "Set this flag to print out the contents of the id_token. Useful for inspecting claims")
+	loginCmd.Flags().BoolVar(&sendAccessTokenArg, "send-access-token", false, "Set this flag to send the Access Token as well as the PK Token in the SSH cert. The Access Token is used to call the userinfo endpoint to get claims not included in the ID Token")
+	loginCmd.Flags().StringVar(&providerArg, "provider", "", "OpenID Provider specification in the format: <issuer>,<client_id> or <issuer>,<client_id>,<client_secret> or <issuer>,<client_id>,<client_secret>,<scopes>")
+	loginCmd.Flags().BoolVarP(&printKeyArg, "print-key", "p", false, "Print private key and SSH cert instead of writing them to the filesystem")
+	loginCmd.Flags().StringVarP(&keyPathArg, "private-key-file", "i", "", "Path where private keys is written")
+	loginCmd.Flags().StringVar(&remoteRedirectURIArg, "remote-redirect-uri", "", "Remote redirect URI used for non-localhost redirects. This is an advanced option for embedding opkssh in server-side logic.")
+	loginCmd.Flags().VarP(enumflag.New(&keyTypeArg, "Key Type", map[commands.KeyType][]string{commands.ECDSA: {commands.ECDSA.String()}, commands.ED25519: {commands.ED25519.String()}}, enumflag.EnumCaseInsensitive), "key-type", "t", "Type of key to generate")
 	rootCmd.AddCommand(loginCmd)
 
 	readhomeCmd := &cobra.Command{
 		SilenceUsage: true,
-		Use:          "readhome <PRINCIPAL>",
+		Use:          "readhome <principal>",
 		Short:        "Read the principal's home policy file",
-		Long: `Read the principal's policy file (/home/<PRINCIPAL>/.opk/auth_id).
+		Long: `Read the principal's policy file (/home/<principal>/.opk/auth_id).
 
 You should not call this command directly. It is called by the opkssh verify command as part of the AuthorizedKeysCommand process to read the user's policy  (principals) home file (~/.opk/auth_id) with sudoer permissions. This allows us to use an unprivileged user as the AuthorizedKeysCommand user.
 `,
@@ -195,9 +238,10 @@ You should not call this command directly. It is called by the opkssh verify com
 	}
 	rootCmd.AddCommand(readhomeCmd)
 
+	var serverConfigPathArg string
 	verifyCmd := &cobra.Command{
 		SilenceUsage: true,
-		Use:          "verify <PRINCIPAL> <CERT> <KEY_TYPE>",
+		Use:          "verify <principal> <cert> <key_type>",
 		Short:        "Verify an SSH key (used by sshd AuthorizedKeysCommand)",
 		Long: `Verify extracts a PK token from a base64-encoded SSH certificate and verifies it against policy. It expects an allowed provider file at /etc/opk/providers and a user policy file at either /etc/opk/auth_id or ~/.opk/auth_id.
 
@@ -221,10 +265,10 @@ Verification checks performed:
 If all checks pass, Verify authorizes the SSH connection.
 
 Arguments:
-  PRINCIPAL    Target username.
-  CERT         Base64-encoded SSH certificate.
-  KEY_TYPE     SSH certificate key type (e.g., ecdsa-sha2-nistp256-cert-v01@openssh.com)`,
-		Args:    cobra.ExactArgs(3),
+  principal    Target username.
+  cert         Base64-encoded SSH certificate.
+  key_type     SSH certificate key type (e.g., ecdsa-sha2-nistp256-cert-v01@openssh.com)`,
+		Args:    cobra.MinimumNArgs(3),
 		Example: `  opkssh verify root <base64-encoded-cert> ecdsa-sha2-nistp256-cert-v01@openssh.com`,
 		RunE: func(cmd *cobra.Command, args []string) error {
 			ctx := context.Background()
@@ -250,10 +294,10 @@ Arguments:
 			userArg := args[0]
 			certB64Arg := args[1]
 			typArg := args[2]
+			extraArgs := args[3:]
 
 			providerPolicyPath := "/etc/opk/providers"
 			providerPolicy, err := policy.NewProviderFileLoader().LoadProviderPolicy(providerPolicyPath)
-
 			if err != nil {
 				log.Println("Failed to open /etc/opk/providers:", err)
 				return err
@@ -268,11 +312,12 @@ Arguments:
 				return err
 			}
 
-			v := commands.VerifyCmd{
-				PktVerifier: *pktVerifier,
-				CheckPolicy: commands.OpkPolicyEnforcerFunc(userArg),
+			v := commands.NewVerifyCmd(*pktVerifier, commands.OpkPolicyEnforcerFunc(userArg), serverConfigPathArg)
+			if err := v.ReadFromServerConfig(); err != nil {
+				log.Println("Failed to set environment variables in config:", err)
 			}
-			if authKey, err := v.AuthorizedKeysCommand(ctx, userArg, typArg, certB64Arg); err != nil {
+
+			if authKey, err := v.AuthorizedKeysCommand(ctx, userArg, typArg, certB64Arg, extraArgs); err != nil {
 				log.Println("failed to verify:", err)
 				return err
 			} else {
@@ -283,8 +328,142 @@ Arguments:
 			}
 		},
 	}
+	verifyCmd.Flags().StringVar(&serverConfigPathArg, "config-path", "/etc/opk/config.yml", "Path to the server config file. Default: /etc/opk/config.yml.")
 	rootCmd.AddCommand(verifyCmd)
 
+	auditCmd := &cobra.Command{
+		SilenceUsage: true,
+		Use:          "audit",
+		Short:        "Validate policy file entries against provider definitions",
+		Long: `Audit validates all entries in /etc/opk/auth_id and ~/.opk/auth_id against the provider definitions in /etc/opk/providers. For complete audit details use the --json flag. Returns a non-zero exit code if any warnings or errors are found.
+
+The audit command checks that:
+  - Each issuer in policy files is defined in the providers file
+  - The protocol (http:// or https://) exactly matches between policy and provider files
+  - The auth_id policy files do not throw parsing errors
+
+Results are reported with the following status:
+  SUCCESS  - Entry is valid
+  WARNING  - Entry is valid but may cause problems
+  ERROR    - Entry has issues (missing provider, protocol mismatch, etc.)
+
+Exit code: 0 if all entries are valid, 1 if any warnings or errors are found.`,
+		Example: `  opkssh audit`,
+		Args:    cobra.NoArgs,
+		RunE: func(cmd *cobra.Command, args []string) error {
+			audit := commands.NewAuditCmd(os.Stdout, os.Stderr)
+
+			// Apply command-line flags
+			providersFile, _ := cmd.Flags().GetString("providers-file")
+			if providersFile != "" {
+				audit.ProviderPath = providersFile
+			}
+
+			policyFile, _ := cmd.Flags().GetString("policy-file")
+			if policyFile != "" {
+				audit.PolicyPath = policyFile
+			}
+
+			skipUser, _ := cmd.Flags().GetBool("skip-user-policy")
+			audit.SkipUserPolicy = skipUser
+
+			audit.JsonOutput, _ = cmd.Flags().GetBool("json")
+			return audit.Run(Version)
+		},
+	}
+
+	auditCmd.Flags().String("providers-file", "/etc/opk/providers", "Path to providers file")
+	auditCmd.Flags().String("policy-file", "/etc/opk/auth_id", "Path to policy file")
+	auditCmd.Flags().Bool("skip-user-policy", false, "Skip auditing user policy file (~/.opk/auth_id)")
+	auditCmd.Flags().BoolP("json", "j", false, "Output complete audit results in JSON")
+
+	rootCmd.AddCommand(auditCmd)
+
+	clientCmd := &cobra.Command{
+		Use:     "client [subcommand]",
+		Short:   "Interact with client configuration",
+		Example: `  opkssh client provider list`,
+		Args:    cobra.ExactArgs(0),
+	}
+
+	providerCmd := &cobra.Command{
+		Use:     "provider [subcommand]",
+		Short:   "Interact with provider configuration",
+		Example: `  opkssh client provider list`,
+		Args:    cobra.ExactArgs(0),
+	}
+
+	providerListCmd := &cobra.Command{
+		Use:     "list",
+		Short:   "List configured providers",
+		Example: `  opkssh client provider list`,
+		Args:    cobra.ExactArgs(0),
+		RunE: func(cmd *cobra.Command, args []string) error {
+			client_config, err := config.GetClientConfigFromFile(configPathArg, afero.NewOsFs())
+
+			if err != nil {
+				log.Fatal("Unable to load providers. ", err)
+			}
+
+			isTTY := term.IsTerminal(int(os.Stdout.Fd()))
+
+			var w *tabwriter.Writer
+			if isTTY {
+				// Nice aligned table for TTY output
+				w = tabwriter.NewWriter(os.Stdout, 0, 0, 2, ' ', 0)
+				fmt.Fprintln(w, "Alias\tIssuer")
+				fmt.Fprintln(w, "-----\t------")
+			} else {
+				// Simpler formatting for non-TTY (e.g., when piping to a file)
+				w = tabwriter.NewWriter(os.Stdout, 0, 0, 1, ' ', tabwriter.DiscardEmptyColumns)
+			}
+
+			for _, p := range client_config.Providers {
+				for _, alias := range p.AliasList {
+					fmt.Fprintf(w, "%s\t%s\n", alias, p.Issuer)
+				}
+			}
+			w.Flush()
+
+			// and lets check it can be loaded into a map, after we print the contents
+			if _, err = config.CreateProvidersMap(client_config.Providers); err != nil {
+				log.Fatal("Unable to parse providers. ", err)
+			}
+
+			return nil
+		},
+	}
+
+	providerListCmd.Flags().StringVar(&configPathArg, "config-path", "", "Path to the client config file. Default: ~/.opk/config.yml on linux and %APPDATA%\\.opk\\config.yml on windows.")
+
+	providerCmd.AddCommand(providerListCmd)
+
+	clientCmd.AddCommand(providerCmd)
+
+	rootCmd.AddCommand(clientCmd)
+
+	// genDocsCmd is a hidden command used as a helper for generating our
+	// command line reference documentation.
+	genDocsCmd := &cobra.Command{
+		Use:    "gendocs <output_dir>",
+		Hidden: true,
+		Args:   cobra.MaximumNArgs(1),
+		RunE: func(cmd *cobra.Command, args []string) error {
+			path := "./docs/cli/"
+			if len(args) > 1 {
+				path = args[1]
+			}
+
+			err := os.MkdirAll(path, 0775)
+			if err != nil {
+				return err
+			}
+
+			return doc.GenMarkdownTree(rootCmd, path)
+		},
+	}
+	rootCmd.AddCommand(genDocsCmd)
+
 	err := rootCmd.Execute()
 	if err != nil {
 		return 1
@@ -310,22 +489,18 @@ func printConfigProblems() {
 // system running the verifier is greater than or equal to 8.1;
 // if not then prints a warning
 func checkOpenSSHVersion() {
-
-	// Redhat/centos does not recognize `sshd -V` but does recognize `ssh -V`
-	// Ubuntu recognizes both
-	cmd := exec.Command("ssh", "-V")
-	output, err := cmd.CombinedOutput()
-	if err != nil {
-		log.Println("Warning: Error executing ssh -V:", err)
+	version := sysdetails.GetOpenSSHVersion()
+	if version == "" {
+		log.Println("Warning: Could not determine OpenSSH version")
 		return
 	}
 
-	if ok, _ := isOpenSSHVersion8Dot1OrGreater(string(output)); !ok {
+	if ok, _ := isOpenSSHVersion8Dot1OrGreater(version); !ok {
 		log.Println("Warning: OpenPubkey SSH requires OpenSSH v. 8.1 or greater")
 	}
 }
 
-func isOpenSSHVersion8Dot1OrGreater(opensshVersion string) (bool, error) {
+func isOpenSSHVersion8Dot1OrGreater(opensshVersionStr string) (bool, error) {
 	// To handle versions like 9.9p1; we only need the initial numeric part for the comparison
 	re, err := regexp.Compile(`^(\d+(?:\.\d+)*).*`)
 	if err != nil {
@@ -333,8 +508,8 @@ func isOpenSSHVersion8Dot1OrGreater(opensshVersion string) (bool, error) {
 		return false, err
 	}
 
-	opensshVersion = strings.TrimPrefix(
-		strings.Split(opensshVersion, ", ")[0],
+	opensshVersion := strings.TrimPrefix(
+		strings.Split(opensshVersionStr, ", ")[0],
 		"OpenSSH_",
 	)
 
@@ -345,9 +520,10 @@ func isOpenSSHVersion8Dot1OrGreater(opensshVersion string) (bool, error) {
 		return false, errors.New("invalid OpenSSH version")
 	}
 
-	version := matches[1]
-
-	if version >= "8.1" {
+	version := "v" + matches[1] // semver requires that version strings start with 'v'
+	// OpenSSH doesn't use semantic versioning, but does use major.minor which after stripping the patch version can be compared using semver
+	if semver.Compare(version, "v8.1.0") >= 0 {
+		// if version is greater than or equal to v8.1.0
 		return true, nil
 	}
 
